# frozen_string_literal: true

module Bulkrax
  class RerunImportersController < ApplicationController
    before_action :set_importer, only: [:rerun_failed_entries, :marked_as_complete]

    def rerun_failed_entries
      current_run = @importer.last_run

      if current_run&.failed_records&.positive?
        ScheduleRelationshipsJob.set(wait: 5.minutes).perform_later(importer_id: @importer.id)
        @importer.status_info('Pending')

        @importer.entries.each do |entry|
          next if entry.status == 'Complete'

          type = if entry.raw_metadata['model'] == 'CrcDataset'
                   'Work'
                 else
                   entry.raw_metadata['model']
                 end

          entry.status_info('Pending')
          current_run.decrement!(:failed_records)
          current_run.increment!(:enqueued_records)

          if type == 'Collection'
            current_run.decrement!(:failed_collections)
          elsif type == 'Work'
            current_run.decrement!(:failed_works)
          elsif type == 'FileSet'
            current_run.decrement!(:failed_file_sets)
          elsif type == 'ComplexSubject'
            current_run.decrement!(:failed_complex_subjects)
          elsif type == 'ComplexSession'
            current_run.decrement!(:failed_complex_sessions)
          elsif type == 'ComplexModality'
            current_run.decrement!(:failed_complex_modalities)
          end

          "Bulkrax::CrcDataset::Import#{type}Job".constantize.send(
            entry.parser.perform_method,
            entry.id,
            @importer.importer_runs.last.id
          )
        end

        current_run.save

      elsif current_run && @importer.parser.total.positive? && current_run.processed_records.zero?
        Bulkrax::ImporterJob.send(
          @importer.parser.perform_method,
          @importer.id
        )
      end

      redirect_to importers_path
    end

    def marked_as_complete
      @importer.last_run.update(enqueued_records: 0)
      @importer.status_info('Complete')

      redirect_to importers_path
    end

    private

    def set_importer
      @importer = Importer.find(params[:importer_id])
    end
  end
end
