FactoryBot.define do

  factory :crc_dataset do
    transient do
      user { create(:user) }
      # Set to true (or a hash) if you want to create an admin set
      with_admin_set { false }
    end

    title { ["CRC dataset"] }
    access_control

    trait :open do
      visibility { 'open' }
      title { ["Open CRC dataset"] }
    end

    trait :authenticated do
      visibility { 'authenticated' }
      title { ["Authenticated CRC dataset"] }
    end

    trait :embargo do
      visibility { 'embargo' }
      title { ["Embargo CRC dataset"] }
    end

    trait :lease do
      visibility { 'lease' }
      title { ["Leased CRC dataset"] }
    end

    trait :restricted do
      visibility { 'restricted' }
      title { ["Restricted CRC dataset"] }
    end

    trait :with_folder_name do
      folder_name { ["experiment_folder"] }
    end

    trait :with_experiment_description do
      experiment_description { ["Experiment-Description-123"] }
    end

    trait :with_crc_resource_type do
      crc_resource_type { "Simulated" }
    end

    trait :with_coverage do
      coverage { "Bochum" }
    end

    trait :with_approval_number do
      approval_number { "A1232" }
    end

    trait :with_extra_information do
      extra_information { ["Extra information"] }
    end

    trait :with_modality do
      modality { ["Behaviour"] }
    end

    trait :with_keyword do
      keyword { ['Keyword-123'] }
    end

    trait :with_keyword_seq do
      sequence(:keyword) { |n| ["Keyword-#{n}"] }
    end

    trait :with_subject do
      subject { ['Subject-123'] }
    end

    trait :with_subject_seq do
      sequence(:subject) { |n| ["Subject-#{n}"] }
    end

    trait :with_language do
      language { ['Faroese'] }
    end

    trait :with_publisher do
      publisher { ['Publisher-123'] }
    end

    trait :with_resource_type do
      resource_type { ['Resource-Type-123'] }
    end

    trait :with_article_resource_type do
      resource_type { ['Article'] }
    end

    trait :with_doi do
      doi { "1234-1567" }
    end

    trait :with_complex_person do
      complex_person_attributes {
        [{
           name: 'Anamika',
           role: ['operator'],
           orcid: '123456',
           affiliation: 'University',
         }]
      }
    end

    trait :with_complex_author do
      complex_person_attributes {
        [{
           name: 'Anamika',
           role: ['author'],
           orcid: '123456',
           affiliation: 'University',
         }]
      }
    end

    trait :with_detailed_complex_author do
      complex_person_attributes {
        [{
           name: 'Anamika',
           first_name: 'First name',
           last_name: 'Last name',
           email: 'a@example.com',
           role: ['author'],
           orcid: '23542345234',
           affiliation: 'My org'
         }]
      }
    end

    trait :with_detailed_complex_people do
      complex_person_attributes {
        [{
           name: 'Anamika',
           first_name: 'First name',
           last_name: 'Last name',
           role: ['author'],
           orcid: '123456',
           affiliation: 'University',
         },
         {
           name: 'Cee Jay',
           first_name: 'First name',
           last_name: 'Last name',
           role: ['editor'],
           orcid: '112233445566',
           affiliation: 'My journal org'
         }]
      }
    end

    trait :with_complex_date do
      complex_date_attributes {
        [{
           date: '2022-10-28',
           description: 'Recorded'
         }]
      }
    end

    trait :with_complex_identifier do
      complex_identifier_attributes {
        [{
           identifier: '10.0.1111',
           scheme: 'DOI'
         }]
      }
    end

    trait :with_complex_relation do
      complex_relation_attributes {
        [{
           title: 'A relation label',
           url: 'http://example.com/relation',
           complex_identifier_attributes: [{
                                             identifier: ['info:hdl/4263537/400'],
                                             scheme: 'identifier persistent'
                                           }],
           relationship: 'isNewVersionOf'
         }]
      }
    end

    trait :with_rights do
      rights_statement {
        [
          'http://creativecommons.org/publicdomain/zero/1.0/'
        ]
      }
    end

    trait :with_complex_funding_reference do
      complex_funding_reference_attributes {
        [{
           funder_identifier: 'f1234',
           funder_name: 'Bank',
           award_number: 'a1234',
           award_uri: 'http://example.com/a1234',
           award_title: 'No free lunch'
         }]
      }
    end
  end
end
